/* eslint-disable @typescript-eslint/no-empty-function */
/* eslint-disable @typescript-eslint/no-unused-vars */
// eslint-disable-next-line max-classes-per-file
import focusFirst from "./helpers/focusFirst";
import nestedProperty from "./helpers/nestedProperty";
import observer from "./helpers/observer";
import onEvent from "./helpers/onEvent";
import { WUPcssHidden, WUPcssBtnIcon } from "./styles";
// theoretically such single appending is faster than using :host inside shadowComponent
const appendedStyles = new Set();
const appendedRootStyles = new Set();
let lastUniqueNum = 0;
// Cached data
const allObservedOptions = new WeakMap();
const allMappedAttrs = new WeakMap();
export var AttributeTypes;
(function (AttributeTypes) {
    /** Value presented by `true`, `false` or '' (empty string) */
    AttributeTypes[AttributeTypes["bool"] = 0] = "bool";
    AttributeTypes[AttributeTypes["number"] = 1] = "number";
    AttributeTypes[AttributeTypes["string"] = 2] = "string";
    AttributeTypes[AttributeTypes["reference"] = 3] = "reference";
    AttributeTypes[AttributeTypes["parsedObject"] = 4] = "parsedObject";
    AttributeTypes[AttributeTypes["parseCustom"] = 5] = "parseCustom";
    /** Element accessed via `document.querySelector` */
    AttributeTypes[AttributeTypes["selector"] = 6] = "selector";
})(AttributeTypes || (AttributeTypes = {}));
window.__wupln = (s) => s;
/** Basic abstract class for every component in web-ui-pack */
export default class WUPBaseElement extends HTMLElement {
    /** Returns this.constructor // watch-fix: https://github.com/Microsoft/TypeScript/issues/3841#issuecomment-337560146 */
    #ctr = this.constructor;
    /** Register control in the web to allow to use */
    static $use() {
        // WARN: this is method required but must be empty for webpack optimization (see sideEffects)
    }
    /** Reference to global style element used by web-ui-pack */
    static get $refStyle() {
        return window.WUPrefStyle || null;
    }
    static set $refStyle(v) {
        window.WUPrefStyle = v || undefined;
    }
    /** StyleContent related to component */
    static get $style() {
        return "";
    }
    /** StyleContent related to component & inherited components */
    static get $styleRoot() {
        return `:root {
          --base-focus: #00778d;
          --base-btn-bg: #009fbc;
          --base-btn-text: #fff;
          --base-btn-focus: #005766;
          --base-btn2-bg: #6c757d;
          --base-btn2-text: #fff;
          --base-btn3-bg: none;
          --base-btn3-text: inherit;
          --base-sep: #e4e4e4;
          --base-margin: 20px;
          --border-radius: 6px;
          --anim-t: 200ms;
          --anim: var(--anim-t) cubic-bezier(0, 0, 0.2, 1) 0ms;
          --icon-hover-r: 30px;
          --icon-hover-bg: #0001;
          --icon-focus-bg: #0000001a;
          --icon-size: 14px;
          --menu-hover-text: inherit;
          --menu-hover-bg: #f1f1f1;
        }
        [wupdark] {
          --base-btn-focus: #bdbdbd;
          --base-sep: #141414;
          --icon: #fff;
          --icon-hover-bg: #fff1;
          --icon-focus-bg: #fff2;
          --scroll: #fff2;
          --scroll-hover: #fff3;
          --menu-hover-text: inherit;
          --menu-hover-bg: #222a36;
        }`;
    }
    /** Get unique id for html elements; Every getter returns new id */
    static get $uniqueId() {
        return `wup${++lastUniqueNum}`;
    }
    /** Returns default class name for visually hidden element */
    static get classNameHidden() {
        return "wup-hidden";
    }
    /** Returns default class name for buttons with icons */
    static get classNameBtnIcon() {
        return "wup-icon";
    }
    /** Returns map-type based on value */
    static mapAttribute(value) {
        if (value == null || value === "auto") {
            return 3 /* AttributeTypes.reference */;
        }
        switch (typeof value) {
            case "boolean":
                return 0 /* AttributeTypes.bool */;
            case "number":
                return 1 /* AttributeTypes.number */;
            case "string":
                return 2 /* AttributeTypes.string */;
            // case "object": return AttributeTypes.reference;
            // case "bigint": - not supported
            // case "symbol": - not supported
            default:
                return 3 /* AttributeTypes.reference */;
        }
    }
    /** Returns map-model based on $defaults for mapping attributes & options */
    static get mappedAttributes() {
        const o = {};
        const def = this.$defaults;
        Object.keys(def).forEach((k) => {
            const attr = k.toLowerCase(); // attributes exists only in lowerCase
            o[attr] = {
                type: this.mapAttribute(def[k]),
                prop: k,
            };
        });
        return o;
    }
    /** Array of options names to listen for changes; @returns `undefined` if need to observe for every option
     * @defaultValue every option from $defaults` */
    static get observedOptions() {
        return null;
    }
    /** Array of attribute names to listen for changes
     * @defaultValue every option from $defaults (in lowerCase) */
    static get observedAttributes() {
        return (this.observedOptions || Object.keys(this.$defaults)).map((k) => `w-${k.toLowerCase()}`);
    }
    /** Global default options applied to every element. Change it to configure default behavior OR use `element.$options` to change per item */
    static $defaults = {};
    /** Used to clone single value (from defaults) */
    static cloneValue(v) {
        if (v == null) {
            return v;
        }
        switch (typeof v) {
            case "function":
                return v;
            case "object":
                if (Array.isArray(v)) {
                    return [...v];
                }
                if (v instanceof HTMLElement) {
                    return v;
                }
                return { ...v };
            default:
                return v;
        }
    }
    /** Used to clone defaults to options on init; override it to clone  */
    static cloneDefaults() {
        return { ...this.$defaults };
    }
    /** Merge options with $defaults; Object.assign merges values 'undefined' by the method replace undefined with defaults */
    static mergeDefaults(opts) {
        const def = this.$defaults;
        Object.keys(def).forEach((k) => {
            if (opts[k] == null) {
                opts[k] = this.cloneValue(def[k]);
            }
        });
        return opts;
    }
    /** Raw part of $options for internal usage (.$options is Proxy object and better avoid useless extra-calles via Proxy) */
    // @ts-expect-error - TS doesn't see that init happens via constructor.$options = null;
    _opts; // = objectClone(this.#ctr.$defaults) as TOptions;
    /* Observed part of $options */
    #optsObserved;
    #removeObserved;
    get $options() {
        // return observed options
        if (this.$isReady) {
            if (!this.#optsObserved) {
                // get from cache
                let o = allObservedOptions.get(this.#ctr);
                if (o === undefined) {
                    const arr = this.#ctr.observedOptions;
                    o = arr ? new Set(arr) : null;
                    allObservedOptions.set(this.#ctr, o);
                }
                const watched = o;
                if (!watched?.size && watched !== null) {
                    return this._opts;
                }
                // cast to observed only if option was retrieved: to optimize init-performance
                this.#optsObserved = observer.make(this._opts, { excludeNested: true });
                this.#removeObserved = observer.onChanged(this.#optsObserved, (e) => {
                    this.#isReady &&
                        e.props.some((p) => (watched == null ? Object.hasOwn(this.#ctr.$defaults, p) : watched.has(p))) &&
                        this.gotOptionsChanged(e);
                });
            }
            return this.#optsObserved;
        }
        // OR return original options if element no appended to body - in this case we don't need to track for changes
        return this._opts;
    }
    /** Options inherited from `static.$defaults` and applied to element. Use this to change behavior per item OR use `$defaults` to change globally */
    set $options(v) {
        if (this._opts === v) {
            return;
        }
        const prev = this._opts;
        if (!v) {
            v = this.#ctr.cloneDefaults();
        }
        this._opts = v;
        if (this.$isReady) {
            // unsubscribe from previous events here
            this.#removeObserved?.call(this);
            this.#optsObserved = undefined;
            this.#removeObserved = undefined;
            const watched = allObservedOptions.get(this.#ctr);
            if (!watched?.size && watched !== null) {
                return; // don't call event if there is nothing to watchFor
            }
            // shallow comparison with filter watched options
            if (prev.valueOf() !== v.valueOf()) {
                const props = [];
                (watched || Object.keys(this.#ctr.$defaults)).forEach((k) => {
                    if (this._opts[k] !== prev[k]) {
                        props.push(k);
                    }
                });
                props.length && this.gotOptionsChanged({ props, target: this._opts });
            }
        }
    }
    /* Return all prototypes till HTMLElement */
    static findAllProtos(t, protos) {
        const p = Object.getPrototypeOf(t);
        if (p !== HTMLElement.prototype) {
            protos.push(p.constructor);
            // Object.getOwnPropertyNames(p).forEach((s) => {
            //   const k = s as keyof Omit<WUPBaseElement, keyof HTMLElement | "$isReady">;
            //   const desc = Object.getOwnPropertyDescriptor(p, s) as PropertyDescriptor;
            //   if (typeof desc.value === "function" && s !== "constructor") {
            //     this[k] = (this[k] as Function).bind(this);
            //   }
            // });
            this.findAllProtos(p, protos);
        }
        return protos;
    }
    constructor() {
        super();
        this.$options = null;
        if (!this.#ctr.$refStyle) {
            this.#ctr.$refStyle = document.createElement("style");
            /* from https://snook.ca/archives/html_and_css/hiding-content-for-accessibility  */
            this.#ctr.$refStyle.append(`.${this.#ctr.classNameHidden}, [${this.#ctr.classNameHidden}] {${WUPcssHidden}}\r\n`);
            this.#ctr.$refStyle.append(`${WUPcssBtnIcon(`[${this.#ctr.classNameBtnIcon}]`)}\r\n`);
            document.head.prepend(this.#ctr.$refStyle);
        }
        const refStyle = this.#ctr.$refStyle;
        // setup styles
        if (!appendedStyles.has(this.tagName)) {
            appendedStyles.add(this.tagName);
            // NiceToHave refactor to append styles via CDN or somehow else
            const protos = this.#ctr.findAllProtos(this, []);
            protos.reverse().forEach((p) => {
                // append $styleRoot
                if (!appendedRootStyles.has(p)) {
                    appendedRootStyles.add(p);
                    if (Object.prototype.hasOwnProperty.call(p, "$styleRoot")) {
                        const s = p.$styleRoot;
                        s && refStyle.append(s);
                    }
                }
            });
            const c = protos[protos.length - 1].$style;
            refStyle.append(c.replace(/:host/g, `${this.tagName}`));
        }
    }
    /** Returns true if element is appended (result of setTimeout on connectedCallback) */
    get $isReady() {
        return this.#isReady;
    }
    /** Returns if current element or some nested child is active/focused */
    get $isFocused() {
        const a = document.activeElement;
        return this === a || this.includes(a);
    }
    /** Try to focus self or first possible children; returns true if successful */
    focus() {
        delete this._willFocus;
        return focusFirst(this);
    }
    /** Remove focus from element on any nested active element */
    blur() {
        const a = document.activeElement;
        if (a === this) {
            super.blur();
        }
        else if (a instanceof HTMLElement && this.includes(a)) {
            a.blur();
        }
    }
    /** Called when need to parse attribute */
    parse(text) {
        return text;
    }
    /** Clear this to prevent autofocus */
    _willFocus;
    #isReady = false;
    /** Called when element is added to document (after empty timeout - at the end of call stack) */
    gotReady() {
        this.#readyTimeout = undefined;
        this.#isReady = true;
        this._isStopChanges = true;
        this.gotChanges(null);
        this._isStopChanges = false;
        if (this.autofocus || this._opts.autoFocus) {
            this._willFocus = setTimeout(() => this.focus(), 2);
        }
    }
    /** Called when element is removed from document */
    gotRemoved() {
        this.#isReady = false;
        this.dispose();
    }
    /** Called on Init and every time as options/attributes changed */
    gotChanges(propsChanged) {
        this.#ctr.mergeDefaults(this._opts); // WARN during the init it fired twice: 1st: cloned defaults, 2nd: merge defaults here
    }
    /** Called when element isReady and at least one of observedOptions is changed */
    gotOptionsChanged(e) {
        this._isStopChanges = true;
        e.props.forEach((p) => {
            this.removeAttribute(`w-${p}`);
        }); // remove related attributes otherwise impossible to override
        this.gotChanges(e.props);
        this._isStopChanges = false;
    }
    /** Called once on Init */
    gotRender() { }
    #isFirstConn = true;
    #readyTimeout;
    /** Browser calls this method when the element is added to the document */
    connectedCallback() {
        // async requires otherwise attributeChangedCallback doesn't set immediately
        this.#readyTimeout = setTimeout(() => this.gotReady.call(this));
        if (this.#isFirstConn) {
            this.#isFirstConn = false;
            this.gotRender();
        }
    }
    /** Browser calls this method when the element is removed from the document;
     * (can be called many times if an element is repeatedly added/removed) */
    disconnectedCallback() {
        this.#readyTimeout && clearTimeout(this.#readyTimeout);
        this.#readyTimeout = undefined;
        this.gotRemoved();
    }
    /** Prevent gotChanges call during the some attribute or options custom changes */
    _isStopChanges = false;
    #attrTimer;
    #attrChanged;
    /** Called when any of observedAttributes is changed */
    gotAttributeChanged(name, value) {
        if (name.startsWith("w-")) {
            name = name.substring(2);
        }
        // WARN: observedAttribute must return same collection as mappedAttributes
        let map = allMappedAttrs.get(this.#ctr); // try to get from cache first
        if (!map) {
            map = this.#ctr.mappedAttributes;
            allMappedAttrs.set(this.#ctr, map);
        }
        const m = map[name] ?? { type: 0 /* AttributeTypes.bool */, prop: name };
        const isRemoved = value == null;
        const key = m.prop ?? name;
        // eslint-disable-next-line no-nested-ternary
        this._opts[key] = isRemoved
            ? this.#ctr.cloneValue(this.#ctr.$defaults[key]) // value == null when attr is removed, then need to rollback to default
            : m.parse
                ? m.parse(value)
                : this.parseAttr(m.type, value, key, name);
        if (this.#isReady) {
            if (!this.#attrTimer) {
                this.#attrChanged = [];
                this.#attrTimer = setTimeout(() => {
                    this.#attrTimer = undefined;
                    this._isStopChanges = true;
                    this.gotChanges(this.#attrChanged);
                    this._isStopChanges = false;
                    this.#attrChanged = undefined;
                });
            }
            this.#attrChanged.push(key);
        }
    }
    /** Browser calls this method when attrs pointed in observedAttributes is changed */
    attributeChangedCallback(name, oldValue, newValue) {
        !this._isStopChanges && oldValue !== newValue && this.gotAttributeChanged(name, newValue);
    }
    addEventListener(type, listener, options) {
        return super.addEventListener(type, listener, options);
    }
    removeEventListener(type, listener, options) {
        return super.removeEventListener(type, listener, options);
    }
    /* eslint-enable max-len */
    /** Inits customEvent & calls dispatchEvent and returns created event; by default `cancelable: false, bubbles: true`
     * @tutorial Troubleshooting
     * * Default event bubbling: el.event.click > el.onclick >>> parent.event.click > parent.onclick etc.
     * * Custom event bubbling: el.$onclick > el.event.$click >>> parent.event.$click otherwise impossible to stop propagation from on['event] of target directly */
    // @ts-expect-error - somehow TS doesn't allow  Events[K]["detail"] but it works
    fireEvent(type, eventInit = {}) {
        eventInit.cancelable ??= false;
        eventInit.bubbles ??= true;
        const ev = new CustomEvent(type, eventInit);
        let sip = false;
        const isCustom = type.startsWith("$");
        if (isCustom) {
            ev.stopImmediatePropagation = () => {
                sip = true;
                CustomEvent.prototype.stopImmediatePropagation.call(ev);
            };
            const str = type.substring(1, 2).toUpperCase() + type.substring(2);
            this[`$on${str}`]?.call(this, ev);
        }
        !sip && super.dispatchEvent(ev);
        return ev;
    }
    /** Array of removeEventListener() that called on remove */
    disposeLst = [];
    /** Add event listener and remove after component removed; @options.passive=true by default */
    appendEvent(...args) {
        // self-removing when option.once
        if (args[3]?.once) {
            const listener = args[2];
            args[2] = function wrapper(...args2) {
                const v = listener.call(this, ...args2);
                remove();
                return v;
            };
        }
        // @ts-ignore
        const r = onEvent(...args);
        this.disposeLst.push(r);
        const remove = () => {
            const i = this.disposeLst.indexOf(r);
            if (i !== -1) {
                r();
                this.disposeLst.splice(i, 1);
            }
        };
        return remove;
    }
    /** Remove events/functions that was appended */
    dispose() {
        this.disposeLst.forEach((f) => f()); // remove possible previous event listeners
        this.disposeLst.length = 0;
    }
    /** Returns true if el is instance of Node and contains pointed element
     * @tutorial Troubleshooting
     * * if element has position `fixed` or `absolute` then returns false */
    includes(el) {
        return el instanceof Node && this.contains(el);
    }
    /** Returns true if element contains eventTarget or it's eventTarget
     * @tutorial Troubleshooting
     * * if element has position `fixed` or `absolute` then returns false */
    includesTarget(e) {
        return this.itsMe(e.target);
    }
    /** Find parent/self according with callback */
    findParent(callback, options = { bubbleCount: 10 }) {
        let i = options?.bubbleCount || 10; // expected no more 10 parents
        let el = this;
        while (--i) {
            if (callback(el)) {
                return el;
            }
            if (el === document.body) {
                return null;
            }
            el = el.parentElement;
            if (!el) {
                return null;
            }
        }
        return null;
    }
    /** Returns true if contains pointed element or has itself
     * @tutorial Troubleshooting
     * * if element has position `fixed` or `absolute` then returns false */
    itsMe(el) {
        return this === el || (el instanceof Node && this.contains(el));
    }
    /** Returns parsed value according to pointed type OR current value if something wrong;
     * override method for implementation custom parsing OR static method mappedAttributes to redefine map-types */
    parseAttr(type, attrValue, propName, attrName) {
        const prev = this._opts[propName];
        // eslint-disable-next-line default-case
        switch (attrValue) {
            case "false":
                return false;
            // case "":
            case "true":
                return true; // empty attr and 'true' is enable`
            case "auto":
                return attrValue;
        }
        switch (type) {
            case 0 /* AttributeTypes.bool */:
                return attrValue !== "false";
            case 1 /* AttributeTypes.number */: {
                const v = +attrValue;
                if (Number.isNaN(v)) {
                    this.throwError(`Expected number for attribute [${attrName}] but pointed '${attrValue}'`);
                    return prev;
                }
                return v;
            }
            case 3 /* AttributeTypes.reference */: {
                if (!attrValue) {
                    return undefined;
                }
                const v = nestedProperty.get(window, attrValue);
                if (v === undefined) {
                    this.throwError(`Value not found according to attribute [${attrName}] in '${attrValue.startsWith("window.") ? attrValue : `window.${attrValue}`}'`);
                    return prev;
                }
                return v;
            }
            case 4 /* AttributeTypes.parsedObject */: {
                try {
                    return this.parse(attrValue);
                }
                catch (err) {
                    this.throwError(err);
                    return prev;
                }
            }
            case 6 /* AttributeTypes.selector */: {
                if (!attrValue) {
                    return undefined;
                }
                const isPrev = attrValue === "prev";
                let el = isPrev ? this.previousElementSibling : document.querySelector(attrValue);
                if (!el) {
                    setTimeout(() => {
                        el = isPrev ? this.previousElementSibling : document.querySelector(attrValue);
                        if (el) {
                            this._opts[propName] = el;
                        }
                        else {
                            this.throwError(`Element not found according to attribute [${attrName}]='${attrValue}' `);
                        }
                    }); // on init item can be missed and added after
                    return attrValue;
                }
                return el;
            }
            default:
                return attrValue; // string
        }
    }
    /** Remove attr if value falseOrEmpty; set '' or 'true' if true for HTMLELement
     * @param isSetEmpty set if need '' instead of 'value' */
    setAttr(attr, v, isSetEmpty) {
        v ? this.setAttribute(attr, isSetEmpty ? "" : v) : this.removeAttribute(attr);
    }
    /** Remove all children in fastest way */
    removeChildren() {
        // benchmark: https://measurethat.net/Benchmarks/Show/23474/2/remove-all-children-from-dom-element-2
        while (this.firstChild) {
            this.removeChild(this.firstChild);
        }
    }
    // NiceToHave: throwError in 90% must be excluded from prod-build
    /** Throws unhandled error via empty setTimeout */
    throwError(err) {
        const e = new Error(`${this.tagName}. ${err}`, { cause: err });
        setTimeout(() => {
            throw e;
        });
    }
    /** Removes empty attr [style] */
    removeEmptyStyle() {
        !this.getAttribute("style") && this.removeAttribute("style");
    }
}
// NiceToHave: HTML attrs-events like 'onsubmit' & 'onchange'
