import { onEvent } from "../indexHelpers";
import WUPPopupElement from "../popup/popupElement";
import WUPBaseControl from "./baseControl";
import WUPTextControl from "./text";
export var MenuOpenCases;
(function (MenuOpenCases) {
    /** When $openMenu() called programmatically; Don't use it for $options (it's for nested cycle) */
    MenuOpenCases[MenuOpenCases["onManualCall"] = 0] = "onManualCall";
    /** When control got focus via user interaction; ignores case when user changed value by click on item on clearButton
     * to change such behavior update WUPBaseComboControl.prototype.canOpenMenu */
    MenuOpenCases[MenuOpenCases["onFocus"] = 2] = "onFocus";
    /** When control got focus programmatically (via option `autofocus` or when called method 'focus()') */
    MenuOpenCases[MenuOpenCases["onFocusAuto"] = 4] = "onFocusAuto";
    /** When user clicks on control (beside editable not-empty input) */
    MenuOpenCases[MenuOpenCases["onClick"] = 8] = "onClick";
    /** When user clicks on input (by default it's disabled rule to allow user to work with input without popup close/open blinks) */
    MenuOpenCases[MenuOpenCases["onClickInput"] = 16] = "onClickInput";
    /** When user types text */
    MenuOpenCases[MenuOpenCases["onInput"] = 32] = "onInput";
    /** When user presses arrowUp/arrowDown */
    MenuOpenCases[MenuOpenCases["onPressArrowKey"] = 64] = "onPressArrowKey";
})(MenuOpenCases || (MenuOpenCases = {}));
export var MenuCloseCases;
(function (MenuCloseCases) {
    MenuCloseCases[MenuCloseCases["onManualCall"] = 0] = "onManualCall";
    MenuCloseCases[MenuCloseCases["onClick"] = 1] = "onClick";
    MenuCloseCases[MenuCloseCases["onSelect"] = 2] = "onSelect";
    MenuCloseCases[MenuCloseCases["onFocusLost"] = 3] = "onFocusLost";
    MenuCloseCases[MenuCloseCases["OnPressEsc"] = 4] = "OnPressEsc";
    MenuCloseCases[MenuCloseCases["OnPressEnter"] = 5] = "OnPressEnter";
    MenuCloseCases[MenuCloseCases["OnOptionsChange"] = 6] = "OnOptionsChange";
})(MenuCloseCases || (MenuCloseCases = {}));
/** Base abstract form-control for any control with popup-menu (Dropdown, Datepicker etc.) */
export default class WUPBaseComboControl extends WUPTextControl {
    /** Returns this.constructor // watch-fix: https://github.com/Microsoft/TypeScript/issues/3841#issuecomment-337560146 */
    #ctr = this.constructor;
    static get $style() {
        return `${super.$style}
      :host {
        cursor: pointer;
      }
      :host input {
        cursor: text;
      }
      :host input:placeholder-shown,
      :host input:read-only {
        cursor: pointer;
      }
      :host label:after {
        content: "";
        -webkit-mask-image: var(--ctrl-icon-img);
        mask-image: var(--ctrl-icon-img);
      }
      @media not all and (prefers-reduced-motion) {
        :host label:after {
          transition: transform var(--anim);
        }
      }
      :host button[clear] {
        margin: 0;
      }
      :host > [menu] {
        padding: 0;
        max-height: 300px;
        z-index: 8010;
      }`;
    }
    static $defaults = {
        ...WUPTextControl.$defaults,
        validationRules: {
            ...WUPBaseControl.$defaults.validationRules,
        },
        openCase: 8 /* MenuOpenCases.onClick */ | 2 /* MenuOpenCases.onFocus */ | 64 /* MenuOpenCases.onPressArrowKey */,
        readOnlyInput: false,
    };
    /** Fires after popup-menu is opened (after animation finishes) */
    $onOpenMenu;
    /** Fires after popup is closed (after animation finishes) */
    $onCloseMenu;
    #isOpened = false;
    /** Returns if popup-menu is opened */
    get $isOpened() {
        return this.#isOpened;
    }
    /** Close popup-menu
     * @returns Promise resolved by animation time */
    async $closeMenu() {
        await this.goCloseMenu(0 /* MenuCloseCases.onManualCall */);
    }
    /** Open popup-menu
     * @returns Promise resolved resolved by animation time */
    async $openMenu() {
        await this.goOpenMenu(0 /* MenuOpenCases.onManualCall */);
        this.#isOpened && (await this.$refPopup.$open()); // wait for popup open-end
    }
    /** Reference to popupMenu */
    $refPopup;
    renderControl() {
        super.renderControl();
        const i = this.$refInput;
        i.setAttribute("role", "combobox");
        i.setAttribute("aria-haspopup", "listbox");
        i.setAttribute("aria-expanded", false);
        i.focus = this.inputFocus; // assign custom method to detect how focus called
    }
    gotChanges(propsChanged) {
        super.gotChanges(propsChanged);
        this._opts.readOnlyInput
            ? this.$refInput.removeAttribute("aria-autocomplete")
            : this.$refInput.setAttribute("aria-autocomplete", "list");
    }
    gotFormChanges(propsChanged) {
        super.gotFormChanges(propsChanged);
        const isMenuEnabled = !this.$isDisabled && !this.$isReadOnly;
        !isMenuEnabled && this.removePopup();
    }
    setupInputReadonly() {
        this.$refInput.readOnly = this.$isReadOnly || !!this._opts.readOnlyInput;
    }
    /** Returns whether possible to use input-click as dropdown behavior */
    isLockInputDropdown(e) {
        return (!(this._opts.openCase & 16 /* MenuOpenCases.onClickInput */) &&
            e.type === "click" &&
            e.target instanceof HTMLInputElement &&
            !e.target.readOnly &&
            e.target.value !== "");
    }
    /** Override to change open-behavior */
    canOpenMenu(openCase, e) {
        if (this.$isReadOnly || this.$isDisabled) {
            return false;
        }
        if (e && this.isLockInputDropdown(e)) {
            return false; // if input readonly > dropdown behavior otherwise allow to work with input instead of opening window
        }
        const can = !!(this._opts.openCase & openCase) || openCase === 0 /* MenuOpenCases.onManualCall */;
        return can;
    }
    async goOpenMenu(openCase, e) {
        if (this.#isOpened) {
            return this.$refPopup;
        }
        const can = this.canOpenMenu(openCase, e);
        if (!can) {
            return null;
        }
        this.#isOpened = true;
        // this.$hideError(); // it resolves overflow menu vs error
        if (!this.$refPopup) {
            const p = document.createElement("wup-popup");
            this.$refPopup = p;
            p.$options.openCase = 0 /* PopupOpenCases.onManualCall */;
            p.$options.target = this;
            p.$options.offsetFitElement = [1, 1];
            p.$options.minWidthByTarget = true;
            p.$options.placement = [
                WUPPopupElement.$placements.$bottom.$start,
                WUPPopupElement.$placements.$bottom.$end,
                WUPPopupElement.$placements.$top.$start,
                WUPPopupElement.$placements.$top.$end,
                WUPPopupElement.$placements.$bottom.$start.$resizeHeight,
                WUPPopupElement.$placements.$bottom.$end.$resizeHeight,
                WUPPopupElement.$placements.$top.$start.$resizeHeight,
                WUPPopupElement.$placements.$top.$end.$resizeHeight,
            ];
            p.setAttribute("menu", "");
            p.$options.animation = 1 /* PopupAnimations.drawer */;
            const menuId = this.#ctr.$uniqueId;
            const i = this.$refInput;
            i.setAttribute("aria-owns", menuId);
            i.setAttribute("aria-controls", menuId);
            // const wasFcs = this.$isFocused;
            this.renderMenu(p, menuId);
            // WARN: rollback the logic if renderMenu() returns Promise
            // const fcs = this.$isFocused;
            // if (!fcs && wasFcs) {
            //   this.#isOpened = false;
            //   this.removePopup(); // fix case when user waited for loading and moved focus to another
            //   return null;
            // }
            this.appendChild(p); // WARN: it will open onInit
            // eslint-disable-next-line no-promise-executor-return
            await new Promise((res) => setTimeout(res)); // wait for appending to body so size is defined and possible to scroll
        }
        if (!this.#isOpened) {
            return null; // possible when user calls open & close sync
        }
        this.$refPopup.$open().then(() => this.fireEvent("$openMenu", { cancelable: false }));
        this.setAttribute("opened", "");
        this.$refInput.setAttribute("aria-expanded", true);
        // await r; // WARN: it's important don't wait for animation to assign onOpened events fast
        return this.$refPopup;
    }
    /** Override to change close-behavior */
    canCloseMenu(closeCase, e) {
        if (e && this.isLockInputDropdown(e)) {
            return false; // if input readonly > dropdown behavior otherwise allow to work with input instead of opening window
        }
        if (closeCase === 1 /* MenuCloseCases.onClick */) {
            if (!(this._opts.openCase & 8 /* MenuOpenCases.onClick */)) {
                return false;
            }
            if (this.$refPopup.includesTarget(e)) {
                return false; // popupClick
            }
        }
        // always close by focusLost
        // if (closeCase === MenuCloseCases.onFocusLost && !(this._opts.openCase & MenuOpenCases.onFocus)) {
        //   return false;
        // }
        return true;
    }
    _isClosing;
    async goCloseMenu(closeCase, e) {
        if (!this.#isOpened || this._isClosing) {
            return false;
        }
        if (!this.canCloseMenu(closeCase, e)) {
            return false;
        }
        this.#isOpened = false;
        this._isClosing = true;
        await this.$refPopup?.$close();
        delete this._isClosing;
        if (this.#isOpened) {
            return false; // possible when popup opened again during the animation
        }
        // remove popup only by focusOut to optimize resources
        if (closeCase === 3 /* MenuCloseCases.onFocusLost */) {
            this.removePopup();
        }
        this.removeAttribute("opened");
        this.$refInput.setAttribute("aria-expanded", false);
        this.focusMenuItem(null);
        this.selectMenuItem(null);
        setTimeout(() => this.fireEvent("$closeMenu", { cancelable: false }));
        return true;
    }
    /** Current focused menu-item (via aria-activedescendant) */
    _focusedMenuItem;
    /** Focus/resetFocus for item (via aria-activedescendant) */
    focusMenuItem(next) {
        this._focusedMenuItem?.removeAttribute("focused");
        if (next) {
            next.id = next.id || this.#ctr.$uniqueId;
            next.setAttribute("focused", "");
            this.$refInput.setAttribute("aria-activedescendant", next.id);
            this.tryScrollTo(next);
        }
        else {
            this.$refInput.removeAttribute("aria-activedescendant");
        }
        this._focusedMenuItem = next;
    }
    /** Scroll to element if previous scroll is not in processing (debounce by empty timeout) */
    tryScrollTo(el) {
        const scroll = el.scrollIntoViewIfNeeded;
        scroll ? scroll.call(el, false) : el.scrollIntoView({ behavior: "instant" });
    }
    _selectedMenuItem;
    /** Select item (set aria-selected and scroll to) */
    selectMenuItem(next) {
        this._selectedMenuItem?.setAttribute("aria-selected", false);
        if (next) {
            next.setAttribute("aria-selected", true);
            this.tryScrollTo(next);
        }
        this._selectedMenuItem = next;
    }
    gotKeyDown(e) {
        // don't allow to process Esc-key when menu is opened
        const isEscPrevent = this.#isOpened && e.key === "Escape";
        !isEscPrevent && super.gotKeyDown(e);
        if (e.altKey || e.shiftKey || e.ctrlKey) {
            return;
        }
        if (!this.#isOpened) {
            if (this._opts.openCase & 64 /* MenuOpenCases.onPressArrowKey */) {
                if (e.key === "ArrowDown" || e.key === "ArrowUp") {
                    e.preventDefault(); // to prevent parent-scroll
                    this.goOpenMenu(64 /* MenuOpenCases.onPressArrowKey */, e).then(() => this.focusMenuItemByKeydown(e));
                    return;
                }
            }
            return;
        }
        switch (e.key) {
            case "Escape":
                e.preventDefault();
                this.resetInputValue();
                this.goCloseMenu(4 /* MenuCloseCases.OnPressEsc */);
                break;
            case "Enter":
                e.preventDefault();
                {
                    const el = this._focusedMenuItem;
                    setTimeout(() => {
                        const isPrevented = el &&
                            // !el.hasAttribute("disabled") && // looks impossible case
                            !el.dispatchEvent(new MouseEvent("click", { cancelable: true, bubbles: true }));
                        if (!isPrevented) {
                            this.$refInput.value || this.$isRequired
                                ? this.resetInputValue()
                                : this.setValue(undefined, 3 /* SetValueReasons.userInput */, true); // reset only when input not empty otherwise user cleared this manually
                            this.goCloseMenu(5 /* MenuCloseCases.OnPressEnter */);
                        }
                    });
                }
                break;
            case " ":
                break; // don't process " " because it can fire clickEvent on buttons
            default:
                this.focusMenuItemByKeydown(e);
                break;
        }
    }
    gotInput(e, preventValueChange) {
        // gotInput possible on browser-autofill so we need filter check if isFocused
        !this.$isOpened &&
            this._opts.openCase & 32 /* MenuOpenCases.onInput */ &&
            this.$isFocused &&
            this.goOpenMenu(32 /* MenuOpenCases.onInput */);
        if (preventValueChange) {
            this._refHistory?.handleInput(e);
        }
        else {
            super.gotInput(e);
        }
    }
    gotFocus(ev) {
        const arr = super.gotFocus(ev);
        // known issue: _isFocusCall missed when el.focus() + browser-tab not active. Reason: 'focusin' called only when browser-tab gets focus
        const sc = this.$refInput._isFocusCall ? 4 /* MenuOpenCases.onFocusAuto */ : 2 /* MenuOpenCases.onFocus */;
        delete this.$refInput._isFocusCall;
        let isPreventedFromClick = false; // possible when user clicks on btnClear or on item inside input to remove it
        setTimeout(() => this.$isFocused && !isPreventedFromClick && this.goOpenMenu(sc, ev), 1); // filter in 1ms to avoid case when focusin + focusout at the same time
        let clickAfterFocus = true; // prevent clickAfterFocus
        let onInputStartClick = false; // mouseDown>selectText>mouseUp - without filter it closes/opens popup when user tries to select text
        const dsps0 = onEvent(this.$refInput, "mousedown", () => {
            const allow = !this.$refInput.readOnly && !(this._opts.openCase & 16 /* MenuOpenCases.onClickInput */);
            onInputStartClick = allow;
        });
        let isInsideClick = false; // fix when labelOnClick > inputOnClick (twice in loop)
        const dsps1 = onEvent(this, "click", (e) => {
            isPreventedFromClick = e.defaultPrevented;
            const skip = e.defaultPrevented || e.button /* only left click */ || isInsideClick || onInputStartClick;
            if (!skip) {
                !this.#isOpened || clickAfterFocus
                    ? this.goOpenMenu(8 /* MenuOpenCases.onClick */, e)
                    : this.goCloseMenu(1 /* MenuCloseCases.onClick */, e);
            }
            isInsideClick = true;
            setTimeout(() => (isInsideClick = false), 1);
        });
        const dsps2 = onEvent(document, "click", (e) => {
            // close by outside click
            !onInputStartClick && !isInsideClick && this.goCloseMenu(1 /* MenuCloseCases.onClick */, e);
            onInputStartClick = false;
        });
        setTimeout(() => document.addEventListener("pointerdown", () => {
            clickAfterFocus = false;
        }, { once: true, passive: true, capture: true }), 10); // fix 10ms => testcase: point autofocus > remove browser focus (click outside browser) > reload page outside browser > click on ctrl with autofocus: menu blinks and hidden after a time but need to open in this case. Also MenuOpenCases.onFocusAuto isn't detected in this case
        arr.push(dsps0, dsps1, dsps2);
        return arr;
    }
    gotFocusLost() {
        !this.#isOpened && !this._isClosing && this.removePopup(); // otherwise it's removed by hidingMenu
        this.goCloseMenu(3 /* MenuCloseCases.onFocusLost */);
        super.gotFocusLost();
    }
    /** Called when user selected new value from menu and need to close menu */
    selectValue(v, canCloseMenu = true) {
        this.setValue(v, 4 /* SetValueReasons.userSelect */);
        canCloseMenu && setTimeout(() => this.goCloseMenu(2 /* MenuCloseCases.onSelect */)); // without timeout it handles click by listener and opens again
    }
    /** Reset input to currentValue; called on pressing Escape or Enter */
    resetInputValue() {
        if (this._inputError) {
            this.$showError(this._inputError); // case when mask/parse applied and need to open input error without changing
            return;
        }
        const isClear = !this.$refInput.value && !this._selectedMenuItem; // if user cleared input and closed menu - need to clearValue
        isClear
            ? this.setValue(undefined, 2 /* SetValueReasons.clear */)
            : this.setInputValue(this.$value, 4 /* SetValueReasons.userSelect */);
    }
    // @ts-expect-error - because expected string
    setInputValue(v, reason) {
        const p = this.valueToInput(v);
        this.setInputValueDirect(p, reason);
    }
    setInputValueDirect(v, reason) {
        super.setInputValue(v, reason);
    }
    /** Called when popup must be removed (by focus out OR if control removed) */
    removePopup() {
        this.$refPopup?.remove();
        this.$refPopup = undefined;
        this._focusedMenuItem = undefined;
        this._selectedMenuItem = undefined;
        if (this.#isOpened) {
            this.#isOpened = false;
            this.removeAttribute("opened");
            this.$refInput.setAttribute("aria-expanded", false);
            this.$refInput.removeAttribute("aria-activedescendant");
        }
    }
    gotRemoved() {
        this.removePopup();
        super.gotRemoved();
    }
    inputFocus() {
        this._isFocusCall = true;
        HTMLInputElement.prototype.focus.call(this);
        delete this._isFocusCall;
    }
}
// WARN about chaining
/*
 $openMenu().then(()=> console.warn('done'))
 isOpened: true >>> create popup,
 isOpening: true
 ...fetch menu-items (takes time)... >>> render popup with items
 ...animation (takes time)...
 isOpening: false
 >>> console.warn('done')
 open-event
 */
/*
 $closeMenu().then(()=> console.warn('done'))
 isOpened: false
 isClosing: true
 ...animation...
 isClosing: false >>> remove popup if focus out
 >>> console.warn('done')
 close-event
 */
// NiceToHave: option for press-Escape: hideMenu + rollback value to that was before showing OR only hideMenu; now WUPTime.$options.menuButtons changes such behavior
