import WUPTextControl from "./text";
import WUPTextareaInput from "./textarea.input";
WUPTextareaInput.$use();
const tagName = "wup-textarea";
/** Form-control with multiline text-input
 * @example
  const el = document.createElement("wup-textarea");
  el.$options.name = "textarea";
  el.$options.validations = {
    required: true,
    max: 250
  };

  const form = document.body.appendChild(document.createElement("wup-form"));
  form.appendChild(el);
  // or HTML
  <wup-form>
    <wup-textarea w-name="textarea" w-validations="myValidations"/>
  </wup-form>;
 * @tutorial innerHTML @example
 * <label>
 *   <span> // extra span requires to use with icons via label:before, label:after without adjustments
 *      <span contenteditable="true" />
 *      <strong>{$options.label}</strong>
 *   </span>
 *   <button clear/>
 * </label>
 * @tutorial Troubleshooting
 * * known issue: NVDA doesn't read multiline text https://github.com/nvaccess/nvda/issues/13369
 * to resolve it set WUPTextareaControl.$defaults.selectOnFocus = true */
export default class WUPTextareaControl extends WUPTextControl {
    /** Returns this.constructor // watch-fix: https://github.com/Microsoft/TypeScript/issues/3841#issuecomment-337560146 */
    // #ctr = this.constructor as typeof WUPTextareaControl;
    static get $style() {
        return `${super.$style}
        :host strong { top: 1.6em; }
        :host [contenteditable=true] {
          min-height: 4em;
          max-height: 4em;
        }`;
    }
    /** Default options - applied to every element. Change it to configure default behavior */
    static $defaults = {
        ...WUPTextControl.$defaults,
        validationRules: {
            ...WUPTextControl.$defaults.validationRules,
            // WARN: validations min/max must depends only on visible chars
            min: (v, setV, c, r) => WUPTextControl.$defaults.validationRules.min.call(c, v?.replace(/\n/g, ""), setV, c, r),
            max: (v, setV, c, r) => WUPTextControl.$defaults.validationRules.max.call(c, v?.replace(/\n/g, ""), setV, c, r),
        },
    };
    $refInput = document.createElement("wup-areainput");
    renderControl() {
        super.renderControl();
        const { id } = this.$refInput;
        this.$refInput.removeAttribute("id");
        this.$refInput.setAttribute("aria-labelledby", id);
        this.$refTitle.id = id;
    }
    gotChanges(propsChanged) {
        super.gotChanges(propsChanged);
        const o = this._opts;
        delete o.mask;
        delete o.maskholder;
        delete o.prefix;
        delete o.postfix;
    }
    renderPrefix() {
        // not supported
    }
    renderPostfix() {
        // not supported
    }
    // protected override gotBeforeInput(e: WUP.Text.GotInputEvent): void {
    //   super.gotBeforeInput(e);
    //   let data: string | null = null;
    //   switch (e.inputType) {
    //     case "insertLineBreak":
    //     case "insertParagraph":
    //       data = "\n";
    //       this.insertText(data);
    //       break;
    //     default:
    //       console.warn(e.inputType, e.data);
    //       break;
    //   }
    //   if (data) {
    //     e.preventDefault();
    //     this.$refInput.dispatchEvent(new InputEvent("input", { inputType: e.inputType, data }));
    //   }
    // }
    // protected override gotInput(e: WUP.Text.GotInputEvent): void {
    //   super.gotInput(e);
    //   console.warn({ v: this.$refInput.value });
    // }
    // protected insertText(text: string): void {
    //   let range = window.getSelection()!.getRangeAt(0);
    //   range.deleteContents(); // delete all prev selected part
    //   const fr = document.createDocumentFragment();
    //   const content = document.createTextNode(text);
    //   fr.appendChild(content);
    //   // issue: when insertText: and endswith "\nAbc " need to remove last empty space
    //   // issue: when delete an empty space: chrome adds <br>
    //   if (text === "\n" && this.$refInput.selectionEnd === this.$refInput.textContent!.length) {
    //     fr.appendChild(document.createTextNode(" ")); // WARN: otherwise Chrome doesn't go to next line
    //   }
    //   range.insertNode(fr);
    //   // create a new range
    //   range = document.createRange();
    //   range.setStartAfter(content);
    //   range.collapse(true);
    //   // make the cursor there
    //   const sel = window.getSelection()!;
    //   sel.removeAllRanges();
    //   sel.addRange(range);
    //   // autoscroll to cursor
    //   const tempAnchorEl = document.createElement("br");
    //   range.insertNode(tempAnchorEl);
    //   tempAnchorEl.scrollIntoView({ block: "nearest" });
    //   tempAnchorEl.remove(); // remove after scrolling is done
    // }
    gotKeyDown(e) {
        super.gotKeyDown(e);
        if (e.key === "Enter") {
            e.submitPrevented = true;
        }
    }
    gotBeforeInput(e) {
        if (e.inputType.startsWith("format")) {
            // todo need to process this for customHistory in the future
            e.preventDefault(); // prevent Bold,Italic etc. styles until textrich is developed
        }
        else {
            super.gotBeforeInput(e);
        }
        // delete (this.$refInput as unknown as WUPTextareaInput)._cached;
    }
    gotFocusLost() {
        super.gotFocusLost();
        this.setInputValue(this.$value, 3 /* SetValueReasons.userInput */); // update because newLine is replaced
    }
}
// prettify defaults before create
let rr = ["mask", "maskholder", "prefix", "postfix"];
rr.forEach((k) => delete WUPTextareaControl.$defaults[k]);
rr = undefined;
customElements.define(tagName, WUPTextareaControl);
// NiceToHave display of btnClear affects on internal width - is it ok ???
