// Discussion: https://stackoverflow.com/questions/35939886/find-first-scrollable-parent
// best solution: https://stackoverflow.com/questions/4880381/check-whether-html-element-has-scrollbars
/** Returns whether element has position fixed OR not */
function hasFixedPos(el) {
    const p = el.style.position;
    const n = "fixed";
    return p === n || (!p && window.getComputedStyle(el).position === n);
}
/** Find first parent with active scroll X/Y */
export default function findScrollParent(el) {
    const p = el.parentElement;
    if (!p) {
        return null;
    }
    // it means it's changed
    if (p.scrollTop > 0 || p.scrollLeft > 0) {
        return p;
    }
    // try to change and check if it's affected
    let prev = p.scrollTop;
    p.scrollTop += 10; // 10 to fix case when zoom applied
    if (prev !== p.scrollTop) {
        p.scrollTop = prev;
        return p; // WARN: scroll changeable even if css overflow:hidden
    }
    prev = p.scrollLeft;
    p.scrollLeft += 10; // 10 to fix case when zoom applied
    if (prev !== p.scrollLeft) {
        p.scrollLeft = prev;
        return p;
    }
    if (hasFixedPos(p)) {
        return null; // skip search if current element with fixed position
    }
    return findScrollParent(p);
}
/** Find all parents with active scroll X/Y */
export function findScrollParentAll(el) {
    let l = el;
    const arr = [];
    // eslint-disable-next-line no-constant-condition
    while (1) {
        l = findScrollParent(l);
        if (l) {
            arr.push(l);
            if (hasFixedPos(l)) {
                break; // skip search if current element with fixed position
            }
        }
        else
            break;
    }
    return arr.length ? arr : null;
}
