const nestedProperty = {
    /**
     * Sets the value at path of object. If a portion of path doesn’t exist it’s created.
     * nestedProperty.set(obj, "value.nestedValue", 1) as obj.value.nestedValue = 1;
     * @param object The object to modify.
     * @param path The path of the property to set.
     * @param value The value to set.
     */
    set(obj, path, value) {
        const propKeys = path.split(".");
        let key = propKeys[0];
        for (let i = 0; i < propKeys.length - 1; key = propKeys[++i]) {
            if (!obj[key]) {
                obj[key] = {};
            }
            obj = obj[key];
        }
        obj[key] = value;
    },
    /**
     * Gets the property value at path of object.
     * nestedProperty.get(obj, "nestedValue1.nestVal2") returns value from obj.nestedValue1.nestVal2
     * @param object The object to query.
     * @param path The path of the property to get.
     * @param out output object. Point empty {} if you want to get extrachecking hasProp (to define if prop undefined and exists)
     * @return Returns the resolved value.
     */
    get(obj, path, out) {
        const deepKeys = path.split(".");
        let next = obj;
        for (let i = 0; i < deepKeys.length; ++i) {
            if (next == null) {
                break;
            }
            if (out != null) {
                out.hasProp = deepKeys[i] in next;
            }
            next = next[deepKeys[i]];
        }
        return next;
    },
};
export default nestedProperty;
