import onEvent from "./onEvent";
let isMouseDown = false;
let rstEvent;
let rstCnt = 0;
const onMouseUp = [];
function setEvent() {
    ++rstCnt;
    if (rstEvent) {
        return;
    }
    const r1 = onEvent(document, "mousedown", ({ button, defaultPrevented }) => (isMouseDown = !button && !defaultPrevented), // filter only for LeftClick
    { passive: true, capture: true });
    // click fires after mouseup but provides better mechanism
    const r2 = onEvent(document, "click", () => {
        onMouseUp.forEach((f) => f());
        onMouseUp.length = 0;
        isMouseDown = false;
    }, { passive: true, capture: true });
    rstEvent = () => {
        if (--rstCnt === 0) {
            onMouseUp.length = 0;
            isMouseDown = false;
            r1();
            r2();
            rstEvent = undefined;
        }
    };
}
/** Fires when element/children completely lost focus.
 * This event checks next focused/active element and isn't called several times when focus goes between children.
 * @param element HTMLElement to apply `.addEventListener`
 * @param listener Callback invoked on event
 * @param options OnFocusLostOptions
 * @return Callback with `.removeEventListener`. Call it to remove listener
 * */
export default function onFocusLost(element, listener, options) {
    setEvent();
    const remove = () => {
        element.removeEventListener("focusout", focusout);
        rstEvent?.call(element);
    };
    const focusout = (e, isNext) => {
        if (!isNext && isMouseDown) {
            // mouseDown Label > mouseUp Label (without mouseMove) >>> click Label > focusin Input > click Input
            // if mouseDown.target === mouseUp.target >>> click target; if clickTarget tied with input >>> clickInput
            // timeout requires to wait for next document.activeElement to be defined
            onMouseUp.push(() => setTimeout(() => focusout(e, true), options?.debounceMs || 100));
            return;
        }
        const isFocused = (a) => a && (element === a || element.contains(a));
        const isStillFocused = e.relatedTarget instanceof Node && isFocused(e.relatedTarget);
        if (!isStillFocused && !isFocused(document.activeElement)) {
            listener.call(element, e);
            options?.once && remove();
        }
    };
    element.addEventListener("focusout", focusout, { passive: true, ...options, once: false });
    return remove;
}
